<?php

// DON'T CALL THE FILE DIRECTLY USE THIS.
if (!defined('ABSPATH')) {
    exit;
}

// BACKWORDS COMPATABILITY FOR OLDER THAN PHP 5.3.0
if (!defined('__DIR__')) {
    define('__DIR__', dirname(__FILE__));
}

/**
 * common functionality
 */
include_once(plugin_dir_path(__FILE__) . 'common_functions.php');

/**
 * shipment functionality
 */
include_once(plugin_dir_path(__FILE__) . 'shipment_function.php');

class WC_Shipit_Uniwin_Settings
{

    //Initialize admin settings
    public function __construct()
    {
        global $wpdb;
        $this->shipit_uniwin_admin_settings();
    }

    public function shipit_uniwin_admin_settings()
    {
        /**
         * Show menu in the wordpress admin panel
         */

        add_action("admin_menu", "shipit_uniwin_menu");
        if (!function_exists('shipit_uniwin_menu')) {
            function shipit_uniwin_menu()
            {
                add_menu_page(
                    "Shipit", //page_title
                    "Shipit", //menu_title
                    "manage_options", //capability
                    "shipit-settings", //menu_slug
                    "shipit_uniwin_settings_function", //function - Load the setting form in the admin panel.
                    plugin_dir_url(__FILE__) . "images/shipit.png" //icon_url
                );
            }
        }


        /**
         * Load all the css and scripts in the wordpress admin panel 
         */

        add_action('init', 'shipit_uniwin_admin_css_scripts');
        if (!function_exists('shipit_uniwin_admin_css_scripts')) {
            function shipit_uniwin_admin_css_scripts()
            {
                wp_enqueue_style("shipit-uniwin-admin-css", plugin_dir_url(__FILE__) . 'css/shipit-for-woocommerce-admin.css', array(), '');

                wp_enqueue_script("shipit-uniwin-cookie-js", plugin_dir_url(__FILE__) . 'js/jquery.cookie.min.js', array('jquery'), true);

                wp_enqueue_script("shipit-uniwin-jquery-validator-js", plugin_dir_url(__FILE__) . 'js/jquery.validate.min.js', array('jquery'), true);

                wp_enqueue_script("shipit-uniwin-admin-js", plugin_dir_url(__FILE__) . 'js/shipit-for-woocommerce-admin.js', array('jquery'), true);
            }
        }


        /**
         * Update shipit info in woocommerce orders table in admin panel
         */
        add_filter('manage_edit-shop_order_columns', 'shipit_uniwin_custom_shop_order_column', 20);
        function shipit_uniwin_custom_shop_order_column($columns)
        {
            $reordered_columns = array();
            // Inserting columns to a specific location
            foreach ($columns as $key => $column) {
                $reordered_columns[$key] = $column;
                if ($key ==  'order_status') {
                    // Inserting after "Status" column
                    $reordered_columns['Shipit Shipment Details'] = __('Shipit Shipment Details', 'shipit-for-woocommerce');
                    $reordered_columns['Carrier Agent'] = __('Carrier Agent', 'shipit-for-woocommerce');
                    $reordered_columns['Shipit Sync'] = __('Shipit Sync', 'shipit-for-woocommerce');
                }
            }
            return $reordered_columns;
        }

        /**
         * Show shipit info in woocommerce orders table in admin panel
         */

        add_filter('manage_shop_order_posts_custom_column', 'shipit_uniwin_custom_orders_list_column', 20, 2);
        function shipit_uniwin_custom_orders_list_column($column, $post_id)
        {
            $order = wc_get_order($post_id);
            $response_id = get_post_meta($post_id, "wc_shipit_uniwin_shipment_response_id");

            switch ($column) {
                case 'Shipit Shipment Details':
                    if (!empty($response_id)) {
                        $show_response = get_post_meta($post_id, "wc_shipit_uniwin_shipment_response_" . $response_id[0]);
                        if (!empty($show_response)) {
                            $response = $show_response[0];
                            $jsondata = json_decode($response['shipit_response'], true);
                            if (isset($jsondata['status']) && ($jsondata['status'] == 1)) {
                                echo '<p>Status : <b>Success</b></p>';
                                echo '<p>Selected Shipment : <b>' . $response['carrier_name'] . '</b></p>';
                                echo '<p>Tracking No: <b>' . $jsondata['trackingNumber'] . '</b></p>';
                                if (isset($jsondata['proforma']) && !empty($jsondata['proforma'])) {
                                    $custom_docs = get_post_meta($post_id, 'shipit_uniwin_custom_docs');
                                    if (!empty($custom_docs)) {
                                        $custom_docs_url = WC_Shipit_Uniwin_Common_Functions::document_url($post_id, $custom_docs[0]);
                                        echo '<p><a class="button button-primary"  style="margin-bottom: 16px;" target="_blank" href="' . $custom_docs_url . '"><b>Customs Docs</b></a></p>';
                                    }
                                }
                                echo '<a class="button button-primary" target="_blank" href="' . $jsondata['freightDoc'][0] . '"><b>Label</b></a>';
                                echo '<a class="button button-primary" style="margin-left: 16px;" target="_blank" href="' . $jsondata['trackingUrls'][0] . '"><b>Track</b></a>';
                            } else {
                                echo '<p>Status : <b>Fail</b></p>';
                                if (!empty($jsondata['error'])) {
                                    echo '<p>Error : <b>' . $jsondata['error']['message'] . '</b></p>';
                                }
                                echo !empty($jsondata['errorbag']) ? '<p>' . implode(',', $jsondata['errorbag']) . '</p>' : "";
                            }
                        }
                    }
                    break;
                case 'Carrier Agent':
                    $agent_meta_data = get_post_meta($post_id, '_shipit_uniwin_carrier_agent_data');
                    if (!empty($agent_meta_data)) {
                        $agent_data = json_decode($agent_meta_data[0], true);
                        $address_name = $agent_data['carrier'] . " - " . $agent_data['name'] . " , " . $agent_data['address1'];
                        echo "<img width='40px' src='" . $agent_data['carrierLogo'] . "'/>";
                        echo "<p>" . $address_name . "," . $agent_data['city'] . "," . $agent_data['zipcode'] . "</p>";
                    }
                    break;
                case 'Shipit Sync':
                    $added = !empty($show_response) ? 1 : 0;
                    echo "<button class='button button-primary shipit-uniwin-auto-sync' type='button' data-added='" . $added . "' data-id='" . $post_id . "'>Sync</button>";
                    break;
            }
        }



        /**
         * Load the setting form in the admin panel.
         */

        if (!function_exists('shipit_uniwin_settings_function')) {
            function shipit_uniwin_settings_function()
            {
                include_once(plugin_dir_path(__FILE__) . "settings_form.php");
            }
        }

        /**
         * Check if given api key is valid or not  using curl method while submit the address settings form
         */
        function api_is_valid($POST)
        {
            $_POST = $POST;
            $api_url = WC_Shipit_Uniwin_Common_Functions::api_url($_POST['unishipit_api_mode']);
            $checksum = hash("sha512", json_encode(array()) . $_POST['unishipit_api_secret']);
            $url =  $api_url . 'list-methods';
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                "X-SHIPIT-KEY: " . $_POST['unishipit_api_key'],
                "X-SHIPIT-CHECKSUM: " . $checksum
            ));
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "GET");
            $response = curl_exec($ch);
            curl_close($ch);
            $r = json_decode($response, true);
            return $r;
        }

        /**
         * Check if given api key is valid or not  using curl method while submit the address settings form
         */
        function new_user_registration($POST)
        {
            $method =  'register';
            $data = [
                "name" => $POST['reg_uniwin_name'],
                "email" => $POST['reg_uniwin_email'],
                "password" => $POST['reg_uniwin_password'],
                "phone" => "+" . $POST['reg_uniwin_phone'],
                "address" => $POST['reg_uniwin_address'],
                "postcode" => $POST['reg_uniwin_postal'],
                "city" => $POST['reg_uniwin_city'],
                "country" => $POST['reg_uniwin_country'],
                "isCompany" => false,
                "contactPerson" => $POST['reg_uniwin_name'],
                "businessId" => $POST['reg_uniwin_vat']
            ];
            $key = "6Mt1yIapz33QzddyYFKKUYGXEGZopmTX";
            $checksum = hash("sha512", (json_encode($data) . $key));
            $postdata = json_encode($data);
            $api_url = "https://apitest.shipit.ax/v1/"; // Get api url
            $url = $api_url . $method;
            $api_key = "NsN/ufJmMOUJSarH";
            $header = array(
                "Content-Type: application/json",
                "X-SHIPIT-KEY: " . $api_key,
                "X-SHIPIT-CHECKSUM: " . $checksum
            );
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT");
            curl_setopt($ch, CURLOPT_POSTFIELDS, $postdata);
            $response = curl_exec($ch);
            curl_close($ch);
            $r = json_decode($response, true);
            return $r;
        }

        /**
         * Adding Meta container admin shop_order pages.
         */

        add_action('add_meta_boxes', 'shipit_uniwin_order_add_meta_boxes');
        if (!function_exists('shipit_uniwin_order_add_meta_boxes')) {
            function shipit_uniwin_order_add_meta_boxes()
            {
                add_meta_box('mv_other_fields', __('Shipit Uniwin', 'woocommerce'), 'shipit_uniwin_add_other_fields_for_packaging', 'shop_order', 'side', 'core');
            }
        }


        /**
         * Adding Meta field in the meta container admin shop_order pages
         */

        if (!function_exists('shipit_uniwin_add_other_fields_for_packaging')) {
            function shipit_uniwin_add_other_fields_for_packaging()
            {
                global $post;
                if (WC_Shipit_Uniwin_Common_Functions::get_api_key()) {
                    //Calculate weight based on weight unit
                    $weight = WC_Shipit_Uniwin_Common_Functions::weight_calculation($post->ID);

                    include_once(plugin_dir_path(__FILE__) . "meta_box_html.php");
                } else {
                    echo "<h2 class='error'>Please update the api credentials in settings tab.<a href='" . home_url() . "/wp-admin/admin.php?page=shipit-settings'>-> Go to settings tab</a></h2>";
                }
            }
        }

        /**
         * Save the data of the Meta field
         */

        add_action('save_post', 'shipit_uniwin_save_wc_order_meta_box_fields', 10, 1);
        if (!function_exists('shipit_uniwin_save_wc_order_meta_box_fields')) {

            function shipit_uniwin_save_wc_order_meta_box_fields($post_id)
            {
                if (isset($_POST['uniwin_shipit_save'])) {

                    // Check if our nonce is set.
                    if (!isset($_POST['mv_other_meta_field_nonce'])) {
                        return $post_id;
                    }
                    $nonce = $_REQUEST['mv_other_meta_field_nonce'];

                    //Verify that the nonce is valid.
                    if (!wp_verify_nonce($nonce)) {
                        return $post_id;
                    }

                    // Check the user's permissions.
                    if ('page' == $_POST['post_type']) {

                        if (!current_user_can('edit_page', $post_id)) {
                            return $post_id;
                        }
                    } else {

                        if (!current_user_can('edit_post', $post_id)) {
                            return $post_id;
                        }
                    }
                    // --- Its safe for us to save the data ! --- //

                    //create shipment api 
                    $_POST["oid"] = $post_id;

                    $response = WC_Shipit_Uniwin_Shipment_Functions::create_shipment($_POST);
                    // Sanitize user input and update the field in the database.
                    $carrier_name = WC_Shipit_Uniwin_Shipment_Functions::all_shipit_services($_POST['uniwin_shipit_service']);
                    $data = array(
                        'package_data' => json_encode(array("country" => $_POST['uniwin_shipit_country'], "service" => $_POST['uniwin_shipit_service'], "weight" => $_POST['uniwin_shipit_weight'], "length" => $_POST['uniwin_shipit_length'], "width" => $_POST['uniwin_shipit_width'], "height" => $_POST['uniwin_shipit_height'], "parcels" => $_POST['uniwin_shipit_parcel'], "fragile" => isset($_POST['uniwin_shipit_fragile']) ? 1 : 0, "label" => isset($_POST['uniwin_shipit_label']) ? 1 : 0, "order_email" => isset($_POST['uniwin_shipit_order_confirm']) ? 1 : 0)),
                        'created_at' => date('Y-m-d H:i:s'),
                        'order_id' => $post_id,
                        'shipit_response' => json_encode($response),
                        'status' => $response['status'],
                        'carrier_name' => $carrier_name
                    );
                    //Add shipment response to order meta.
                    $unique_key = 1;
                    if (!empty(get_post_meta($post_id, "wc_shipit_uniwin_shipment_response_id"))) {

                        $unique_key = get_post_meta($post_id, "wc_shipit_uniwin_shipment_response_id")[0] + 1;
                        update_post_meta($post_id, 'wc_shipit_uniwin_shipment_response_id', $unique_key);
                    } else {
                        add_post_meta($post_id, 'wc_shipit_uniwin_shipment_response_id', $unique_key, true);
                    }

                    add_post_meta($post_id, 'wc_shipit_uniwin_shipment_response_' . $unique_key, $data, true);
                    if ($response['status'] == 1) {

                        $labels = array();
                        $printing_shiping_label = WC_Shipit_Uniwin_Common_Functions::printing_shiping_label();

                        if ($printing_shiping_label == 1) {
                            // Freight docs
                            $pritinglabels = $response['freightDoc'];
                            foreach ($pritinglabels as $url) {
                                $label = WC_Shipit_Uniwin_Common_Functions::get_printing_label_data($url);
                                if ($label) {
                                    $labels[] = array(
                                        'type' => 'label',
                                        'contents' => $label,
                                    );
                                }
                            }
                            // Proforma
                            if (isset($response['proforma']) && !empty($response['proforma'])) {
                                $proforma = WC_Shipit_Uniwin_Common_Functions::get_printing_label_data($response['proforma']);
                                if ($proforma) {
                                    $labels[] = array(
                                        'type' => 'proforma',
                                        'contents' => $proforma,
                                    );
                                }
                            }
                        }

                        if (!empty($labels)) {
                            // Save labels to the filesystem
                            $i = 1;
                            $documents = array();
                            foreach ($labels as $label) {
                                $document_id = sprintf("%s-%d", $response['orderId'], $i);
                                WC_Shipit_Uniwin_Common_Functions::save_printing_label_document($label['contents'], $document_id);
                                $documents[] = array(
                                    'id' => $document_id,
                                    'type' => $label['type'],
                                );
                                $i++;
                            }
                        }
                        // Print shipping label immediately after creating it
                        $redirect = false;
                        if ($printing_shiping_label == 1) {
                            if (!empty($documents) && count($documents) === 1) {
                                $document = reset($documents);
                                add_post_meta($post_id, 'shipit_uniwin_shipping_label', $document['id'], true);
                                $redirect = WC_Shipit_Uniwin_Common_Functions::document_url($post_id, $document['id']);
                            } else {
                                add_post_meta($post_id, 'shipit_uniwin_shipping_label', $documents[0]['id'], true);
                                add_post_meta($post_id, 'shipit_uniwin_custom_docs', $documents[1]['id'], true);
                                $redirect = WC_Shipit_Uniwin_Common_Functions::document_url($post_id, $documents[0]['id']);
                            }
                        }
                        $update_order_to_completed = WC_Shipit_Uniwin_Common_Functions::update_order_to_completed();
                        if ($update_order_to_completed == 1) {
                            $order_status = WC_Shipit_Uniwin_Common_Functions::set_order_as_completed($post_id);
                        }
                        if (!empty($redirect)) {
                            header('Location:' . $redirect);
                            exit();
                        }
                    }
                }
            }
        }

        /**
         * shipit service price
         */

        add_action('wp_ajax_get_shipit_services_price', 'get_shipit_services_price');
        add_action('wp_ajax_nopriv_get_shipit_services_price', 'get_shipit_services_price');

        if (!function_exists('get_shipit_services_price')) {
            function get_shipit_services_price()
            {

                /**
                 * Fetching prices for services
                 */
                if (!current_user_can('manage_woocommerce')) {
                    die('Permission denied');
                }

                $methods = WC_Shipit_Uniwin_Shipment_Functions::shipment_service_price($_POST);

                include 'admin_price_html.php';

                die;
            }
        }


        /**
         * shipit agent retrive
         */

        add_action('wp_ajax_get_shipit_agent_data', 'get_shipit_agent_data');
        add_action('wp_ajax_nopriv_get_shipit_agent_data', 'get_shipit_agent_data');

        if (!function_exists('get_shipit_agent_data')) {
            function get_shipit_agent_data()
            {

                /**
                 * Fetching prices for services
                 */
                if (!current_user_can('manage_woocommerce')) {
                    die('Permission denied');
                }

                $methods = WC_Shipit_Uniwin_Shipment_Functions::shipment_carrier_agent_data($_POST);

                include 'carrier_agent_html.php';

                die;
            }
        }


        /**
         * save agent data that changed by admin in the popup
         */

        add_action('wp_ajax_save_shipit_agent_data', 'save_shipit_agent_data');
        add_action('wp_ajax_nopriv_save_shipit_agent_data', 'save_shipit_agent_data');
        if (!function_exists('save_shipit_agent_data')) {
            function save_shipit_agent_data()
            {

                /**
                 * Fetching prices for services
                 */
                if (!current_user_can('manage_woocommerce')) {
                    die('Permission denied');
                }

                $response = WC_Shipit_Uniwin_Shipment_Functions::shipment_carrier_agent_save_data($_POST);
                wp_send_json($response);
            }
        }



        /**
         * List the available services based on sender and receiver address
         */

        add_action('wp_ajax_get_shipit_services_for_customer', 'get_shipit_services_for_customer');
        add_action('wp_ajax_nopriv_get_shipit_services_for_customer', 'get_shipit_services_for_customer');

        if (!function_exists('get_shipit_services_for_customer')) {

            function get_shipit_services_for_customer()
            {
                $response = WC_Shipit_Uniwin_Shipment_Functions::receiver_shipment_service_list($_POST);
                wp_send_json($response);
            }
        }


        /**
         * Error Notification
         */
        add_action('admin_notices', 'my_error_notice', 10, 1);
        function my_error_notice($msg)
        {
            if (!empty($msg)) { ?>
                <div class=" error notice is-dismissible">
                    <p><?php echo $msg; ?></p>
                </div>
            <?php
            }
        }



        /**
         * Update Notification
         */
        add_action('admin_update_notices', 'my_update_notice', 10, 1);
        function my_update_notice($msg)
        {
            if (!empty($msg)) {
            ?>
                <div class="updated notice is-dismissible">
                    <p><?php echo $msg; ?></p>
                </div>
            <?php
            }
        }



        /**
         * Get all shipping zone for mapping shipment service
         */

        function shipit_uniwin_get_all_shipping_zones()
        {
            $data_store = WC_Data_Store::load('shipping-zone');
            $raw_zones = $data_store->get_zones();
            if (!empty($raw_zones)) {
                foreach ($raw_zones as $raw_zone) {
                    $zones[] = new WC_Shipping_Zone($raw_zone);
                }
                return $zones;
            }
        }

        /**
         * Get all shipping methods for mapping shipment service
         */

        add_filter('woocommerce_shipping_methods', 'add_Shipit_shipping');
        function add_Shipit_shipping($methods)
        {
            $methods['Shipit_shipping'] = 'Shipit_shipping_Method';
            return $methods;
        }

        /**
         * show shipit agent serivce option in shipping method.
         */

        add_action('woocommerce_shipping_init', 'Shipit_shipping_method');
        function Shipit_shipping_method()
        {
            class Shipit_shipping_Method extends WC_Shipping_Method
            {

                public function __construct($instance_id = 0)
                {
                    $this->id = 'Shipit_shipping';
                    $this->instance_id = absint($instance_id);
                    $this->domain = 'Shipit_shipping';
                    $this->method_title = __('Shipit shipping', $this->domain);
                    $this->title = __('Shipit shipping', $this->domain);
                    $this->supports = array(
                        'shipping-zones',
                        'instance-settings',
                        'instance-settings-modal',
                    );

                    $cost_desc = __('Enter a cost (excl. tax) or sum, e.g. <code>10.00 * [qty]</code>.', 'woo-carrier-agents') . '<br/><br/>' . __('Use <code>[qty]</code> for the number of items.', 'woo-carrier-agents');

                    $get_checkout_is_enabled = WC_Shipit_Uniwin_Common_Functions::get_checkout_is_enabled();
                    if ($get_checkout_is_enabled == 1) {
                        $this->instance_form_fields = array(
                            'title' => array(
                                'title'       => __('Title', 'woocommerce'),
                                'type'        => 'text',
                                'description' => __('This controls the title which the user sees during checkout.', 'woocommerce'),
                                'default'     => 'Shipit Shipping',
                                'desc_tip'    => true,
                            ),
                            'tax_status' => array(
                                'title'         => __('Tax Status', 'woocommerce'),
                                'type'             => 'select',
                                'class'         => 'wc-enhanced-select',
                                'default'         => 'taxable',
                                'options'        => array(
                                    'taxable'     => __('Taxable', 'woocommerce'),
                                    'none'         => _x('None', 'Tax status', 'woocommerce')
                                )
                            ),
                            'cost' => array(
                                'title'         => __('Cost', 'woocommerce'),
                                'type'             => 'text',
                                'placeholder'    => '0',
                                'description'    => $cost_desc,
                                'default'        => '',
                                'desc_tip'        => true,
                                'class'     => 'wpa-field-fixed-rate',
                            ),
                            'service_ids' => array(
                                'title'         => __('Pickup agent search', 'shipit-for-woocommerce'),
                                'type'             => 'multiselect',
                                'class'         => 'wc-enhanced-select',
                                'default'         => '',
                                'options'        => array(
                                    'sbtlfirrex.sbtlfirrex' => "DB Schenker",
                                    'posti.po2103' => "Postipaketti",
                                    'posti.po2331' => "Baltian Postipaketti",
                                    'posti.po2351' => "PickUp Parcel",
                                    'posti.po2711eepp' => "Parcel Connect (EE)",
                                    'posti.po2711ltpp' => "Parcel Connect (LT)",
                                    'posti.po2711lvpp' => "Parcel Connect (LV)",
                                    'mh.mh10' => "Bussipaketti",
                                    'mh.mh20' => "Pikapaketti",
                                    'mh.mh80' => "Lähellä-paketti",
                                    'mh.mh84' => 'XXS-paketti',
                                    'mh.mh95' => 'Euroopan Lähellä-paketti',
                                    'plscm.p19fi' => 'Postnord MyPack',
                                    'plscm.p19fidpd' => 'Postnord MyPack DPD',
                                    'ups.accesp' => 'UPS Express Access Point',
                                    'ups.accespstd' => 'UPS Access Point Standard',
                                    'omni.omnice' => 'Omniva Pickup Point Delivery (EE)',
                                    'omni.omniparcelmachine' => 'Omniva Parcelmachine (EE)',
                                    'dpd.dindpdbaltpickup' => 'DPD Parcelmachine and Pickup Point (EE)',
                                )
                            ),
                        );
                    } else {
                        $this->instance_form_fields = array(
                            'title' => array(
                                'title'       => __('Title', 'woocommerce'),
                                'type'        => 'text',
                                'description' => __('This controls the title which the user sees during checkout.', 'woocommerce'),
                                'default'     => 'Shipit Shipping',
                                'desc_tip'    => true,
                            ),
                            'tax_status' => array(
                                'title'         => __('Tax Status', 'woocommerce'),
                                'type'             => 'select',
                                'class'         => 'wc-enhanced-select',
                                'default'         => 'taxable',
                                'options'        => array(
                                    'taxable'     => __('Taxable', 'woocommerce'),
                                    'none'         => _x('None', 'Tax status', 'woocommerce')
                                )
                            ),
                            'cost' => array(
                                'title'         => __('Cost', 'woocommerce'),
                                'type'             => 'text',
                                'placeholder'    => '0',
                                'description'    => $cost_desc,
                                'default'        => '',
                                'desc_tip'        => true,
                                'class'     => 'wpa-field-fixed-rate',
                            ),
                        );
                    }
                    $this->enabled = $this->get_option('enabled');
                    $this->title = $this->get_option('title');
                    $this->tax_status = $this->get_option('tax_status');
                    $this->cost = $this->get_option('cost');

                    add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
                }

                public function calculate_shipping($package = array())
                {
                    $this->add_rate(array(
                        'id'    => $this->id . $this->instance_id,
                        'label' => $this->title,
                        'cost'  => $this->cost,
                    ));
                }
            }
        }


        /**
         * This function is useful for retrieving the name of a shipping method based on its ID.*
         */

        function get_shipping_name_by_id($chosen_shipping_methods)
        {
            if (preg_match("/\d+/", $chosen_shipping_methods, $instance_id)) {
                $shipping_method = WC_Shipping_Zones::get_shipping_method($instance_id[0]);
                return $shipping_method->get_title();
            }
            return null;
        }



        /**
         * The is a custom function for WooCommerce that adds carrier agents on the checkout page. It first checks if the checkout feature is enabled and then proceeds to check if the current page is the cart page. If it is, the function returns and does not proceed further.  Next, it retrieves the chosen shipping method and its instance ID, method ID, and shipping method instance. It then fetches the service IDs from the shipping method instance settings.  Using the customer's shipping postcode and country, the function sends a request to the API to get the locations of the carrier agents. It then creates a list of options for the carrier agents based on the response received. Finally, it displays the carrier agents as either a dropdown list (select) or radio buttons, depending on the agent style specified in the settings. The function also adds hidden input fields for each carrier agent's location details.This function is hooked to the 'woocommerce_after_shipping_rate' action, which means it will be executed after the shipping rate is displayed on the checkout page.*
         */

        add_action('woocommerce_after_shipping_rate', 'add_carrier_agents_on_checkout', 20, 2);

        function add_carrier_agents_on_checkout($method, $index)
        {
            $get_checkout_is_enabled = WC_Shipit_Uniwin_Common_Functions::get_checkout_is_enabled();
            if ($get_checkout_is_enabled == 1) {
                // Targeting checkout page only:
                if (is_cart()) return; // Exit on cart page

                $instance_id = "";
                $method_id = "";
                $shipping_method_instance = "";

                $chosen_shipping_methods = WC()->session->get("chosen_shipping_methods");
                if ($chosen_shipping_methods[0] === $method->id) {
                    $instance_id = $method->instance_id;
                    $method_id   = $method->method_id; // The shipping method slug
                    $shipping_method_instance = "woocommerce_" . $method_id . "_" . $instance_id . "_settings";
                    $service_ids = get_option($shipping_method_instance);

                    global $woocommerce;
                    if (isset($service_ids['service_ids'])) {
                        if (count($service_ids['service_ids']) > 0) {

                            $postcode = $woocommerce->customer->get_shipping_postcode();
                            $country = $woocommerce->customer->get_shipping_country();
                            if (!empty($postcode) && !empty($country)) {
                                $body = [
                                    'postcode' => $postcode,
                                    'country' => $country,
                                    'serviceId' => $service_ids['service_ids'],
                                    'type' => "PICKUP"
                                ];

                                $api_url = 'agents';

                                $checksum = WC_Shipit_Uniwin_Common_Functions::calculate_checksum($body);
                                $postdata = json_encode($body);
                                $response = WC_Shipit_Uniwin_Common_Functions::curl_all_method($api_url, $checksum, "POST", $postdata);
                                if ($response['status'] == 1 && isset($response['locations'])) {
                                    $options = [];
                                    foreach ($response['locations'] as $location) {
                                        $address_name = $location['carrier'] . " - " . $location['name'] . " , " . $location['address1'];
                                        $options[$location['id']] = $address_name;
                                        foreach ($location as $key => $loc) {
                                            $hidden_name = "carrier_agents:" . $instance_id . ":" . $location['id'] . ":" . $key;
                                            echo "<input type='hidden' name='" . $hidden_name . "' value='" . $loc . "'/>";
                                        }
                                    }
                                }
                            }

                            $field_name = "carrier_agents:" . $instance_id;
                            $agent_style = WC_Shipit_Uniwin_Common_Functions::get_carrier_agent_style();
                            echo '<div class="carrier_agents">';
                            woocommerce_form_field(
                                $field_name,
                                array(
                                    'type'          => ($agent_style == "select") ? "select" : "radio",
                                    'class'         => array('carrier_agents form-row-wide wc-enhanced-select'),
                                    'required'    => true,
                                    'options'     => $options
                                ),
                            );
                            echo '</div>';
                        }
                    }
                }
            }
        }

        /**
         * Save carrier agent data and auto sync in shipment
         */

        add_action('woocommerce_checkout_order_processed', 'save_shipit_carrier_agent_data', 10, 3);
        function save_shipit_carrier_agent_data($order_id, $posted_data, $order)
        {
            $get_checkout_is_enabled = WC_Shipit_Uniwin_Common_Functions::get_checkout_is_enabled();
            if ($get_checkout_is_enabled == 1) {
                $shipping_methods = $order->get_shipping_methods();

                if (empty($shipping_methods)) {
                    return;
                }

                $shipping_method = reset($shipping_methods);
                $instance_id = FALSE;
                if (is_a($shipping_method, 'WC_Order_Item_Shipping')) {
                    if (method_exists($shipping_method, 'get_instance_id')) {
                        $instance_id = $shipping_method->get_instance_id();
                    } else if (method_exists($shipping_method, 'get_method_id')) { // WooCommerce < 3.4
                        $shipping_rate_ids = explode(':', $shipping_method->get_method_id());
                        $shipping_rate_id = $shipping_rate_ids[0];
                        $instance_id = $shipping_rate_ids[1];
                    }
                }

                // No instance ID found, abort
                if (!$instance_id) {
                    return;
                }

                $agent_id = $_REQUEST['carrier_agents:' . $instance_id];
                $agent_data_id = $_REQUEST['carrier_agents:' . $instance_id . ":" . $agent_id . ":id"];
                $agent_data_name = $_REQUEST['carrier_agents:' . $instance_id . ":" . $agent_id . ":name"];
                $agent_data_address1 = $_REQUEST['carrier_agents:' . $instance_id . ":" . $agent_id . ":address1"];
                $agent_data_zipcode = $_REQUEST['carrier_agents:' . $instance_id . ":" . $agent_id . ":zipcode"];
                $agent_data_city = $_REQUEST['carrier_agents:' . $instance_id . ":" . $agent_id . ":city"];
                $agent_data_countryCode = $_REQUEST['carrier_agents:' . $instance_id . ":" . $agent_id . ":countryCode"];
                $agent_data_serviceId = $_REQUEST['carrier_agents:' . $instance_id . ":" . $agent_id . ":serviceId"];
                $agent_data_carrier = $_REQUEST['carrier_agents:' . $instance_id . ":" . $agent_id . ":carrier"];
                $agent_data_carrierLogo = $_REQUEST['carrier_agents:' . $instance_id . ":" . $agent_id . ":carrierLogo"];
                $agent_data = array(
                    "id" => $agent_data_id,
                    "name" => $agent_data_name,
                    "address1" => $agent_data_address1,
                    "zipcode" => $agent_data_zipcode,
                    "city" => $agent_data_city,
                    "countryCode" => $agent_data_countryCode,
                    "serviceId" => $agent_data_serviceId,
                    "carrier" => $agent_data_carrier,
                    "carrierLogo" => $agent_data_carrierLogo

                );
                if (!empty($agent_id)) {
                    update_post_meta($order_id, '_shipit_uniwin_carrier_agent_id', $agent_id);
                    update_post_meta($order_id, "_shipit_uniwin_carrier_agent_data", json_encode($agent_data));
                }

                // AUTO SYNC
                $settings = WC_Shipit_Uniwin_Common_Functions::get_settings_info();
                if (isset($settings->auto_sync) && ($settings->auto_sync == 1)) {
                    // get shipping service
                    global $wpdb;
                    $tblname = $wpdb->prefix . 'shipit_uniwin_shipping_mapping';
                    $shipping_mapped = $wpdb->get_var($wpdb->prepare("SELECT service_id FROM `" . $tblname . "` WHERE shipping_method='$instance_id'"));

                    if (!empty($shipping_mapped)) {
                        //Calculate weight based on weight unit
                        $weight = WC_Shipit_Uniwin_Common_Functions::weight_calculation($order_id);
                        $_post['uniwin_shipit_country'] = $posted_data['shipping_country'];
                        $_post['uniwin_shipit_parcel'] = $settings->pk_parcel;
                        $_post['uniwin_shipit_weight'] = !empty($weight) ? $weight : 0.6;
                        $_post['uniwin_shipit_width'] = $settings->pk_width;
                        $_post['uniwin_shipit_height'] = $settings->pk_height;
                        $_post['uniwin_shipit_length'] = $settings->pk_length;
                        $_post['uniwin_shipit_service'] = $shipping_mapped;
                        $_post['uniwin_shipit_label'] = ($settings->return_label == 1) ? true : false;
                        $_post['uniwin_shipit_fragile'] = ($settings->fragile == 1) ? true : false;
                        $_post['uniwin_shipit_order_confirm'] = ($settings->order_confirm_email == 1) ? true : false;
                        $_post['uniwin_shipit_pickup'] = $agent_id;
                        $_post["oid"] = $order_id;
                        $response = WC_Shipit_Uniwin_Shipment_Functions::create_shipment($_post);
                        // Sanitize user input and update the field in the database.
                        $carrier_name = WC_Shipit_Uniwin_Shipment_Functions::all_shipit_services($shipping_mapped);
                        $data = array(
                            'package_data' => json_encode(array("country" => $posted_data['shipping_country'], "service" => $_post['uniwin_shipit_service'], "weight" => $_post['uniwin_shipit_weight'], "length" => $_post['uniwin_shipit_length'], "width" => $_post['uniwin_shipit_width'], "height" => $_post['uniwin_shipit_height'], "parcels" => $_post['uniwin_shipit_parcel'], "fragile" => isset($_post['uniwin_shipit_fragile']) ? 1 : 0, "label" => isset($_post['uniwin_shipit_label']) ? 1 : 0, "order_email" => isset($_post['uniwin_shipit_order_confirm']) ? 1 : 0)),
                            'created_at' => date('Y-m-d H:i:s'),
                            'order_id' => $order_id,
                            'shipit_response' => json_encode($response),
                            'status' => $response['status'],
                            'carrier_name' => $carrier_name
                        );
                        //Add shipment response to order meta.
                        $unique_key = 1;
                        if (!empty(get_post_meta($order_id, "wc_shipit_uniwin_shipment_response_id"))) {
                            $unique_key = get_post_meta($order_id, "wc_shipit_uniwin_shipment_response_id")[0] + 1;
                            update_post_meta($order_id, 'wc_shipit_uniwin_shipment_response_id', $unique_key);
                        } else {
                            add_post_meta($order_id, 'wc_shipit_uniwin_shipment_response_id', $unique_key, true);
                        }
                        add_post_meta($order_id, 'wc_shipit_uniwin_shipment_response_' . $unique_key, $data, true);
                        if ($response['status'] == 1) {
                            $update_order_to_completed = WC_Shipit_Uniwin_Common_Functions::update_order_to_completed();
                            if ($update_order_to_completed == 1) {
                                $order_status = WC_Shipit_Uniwin_Common_Functions::set_order_as_completed($order_id);
                            }
                        }
                    }
                }
            }
        }

        /**
         * This code adds an action to the WooCommerce thank you page and displays the carrier agent information if enabled. It first checks if the checkout feature is enabled and if the carrier agent information should be displayed on the thank you page. If both conditions are met, it retrieves the agent data from the order metadata and formats it into an HTML string to display on the page.
         */

        add_action('woocommerce_thankyou', 'shipit_uniwin_show_carrier_agent_tq_page', 10, 2);
        function shipit_uniwin_show_carrier_agent_tq_page($order_id)
        {
            $get_checkout_is_enabled = WC_Shipit_Uniwin_Common_Functions::get_checkout_is_enabled();
            if ($get_checkout_is_enabled == 1) {
                $show_agent = WC_Shipit_Uniwin_Common_Functions::show_agent_on_tq_page();
                if ($show_agent == 1) {
                    $agent = get_post_meta($order_id, '_shipit_uniwin_carrier_agent_data');
                    if (!empty($agent)) {
                        $agent_data = json_decode($agent[0], true);
                        $address_name = $agent_data['carrier'] . " - " . $agent_data['name'] . " , " . $agent_data['address1'];
                        $html = "<div class='show_carrier_agent_text'><h4>Pickup Location : </h4>";
                        $html .= "<p>" . $address_name . "</p></div>";
                        echo $html;
                    }
                }
            }
        }


        /**
         * Attach shipment tracking URL to email.
         */

        add_action('woocommerce_email_order_meta', 'attach_tracking_url_to_email', 10, 4);

        function attach_tracking_url_to_email($order, $sent_to_admin = false, $plain_text = false, $email = null)
        {
            $add_to_email = WC_Shipit_Uniwin_Common_Functions::add_track_url_to_email();
            // Targetting specific email notifications
            $email_ids = array('customer_completed_order');
            if ($add_to_email == 1 && isset($email->id)) {
                // Checks if a value exists in an array
                if (in_array($email->id, $email_ids)) {
                    $shipment_info = WC_Shipit_Uniwin_Shipment_Functions::get_shipment_data($order->get_id());
                    if (!empty($shipment_info)) {
                        $sinfo = get_post_meta($order->get_id(), "wc_shipit_uniwin_shipment_response_" . $shipment_info[0])[0];
                        if ($sinfo["status"] == 1) {
                            $shipit_response = json_decode($sinfo["shipit_response"], true);
                            if (!empty($shipit_response['trackingUrls'][0])) {
                                if ($plain_text) {
                                    echo sprintf(__("You can track your order at %s with tracking code %s.\n", 'shipit-for-woocommerce'), $shipit_response['trackingUrls'][0], $shipit_response['trackingNumber']);
                                } else {
                                    echo '<h2>' . __('Tracking', 'shipit-for-woocommerce') . '</h2>';
                                    echo '<p>' . sprintf(__('You can track your order <a href="%s">here</a> with tracking code %s.', 'shipit-for-woocommerce'), $shipit_response['trackingUrls'][0], $shipit_response['trackingNumber']) . '</p>';
                                }
                            }
                        }
                    }
                }
            }
        }

        /**
         * This code adds an action to save custom fields for a product post type. It checks if the outside EU option is enabled and if it is, it saves the values for three custom fields: tariff code, custom description, and custom origin.
         */

        add_action('woocommerce_product_options_shipping', 'add_tariff_orgint_custom_fields');

        function add_tariff_orgint_custom_fields()
        {
            $check_outside_eu = WC_Shipit_Uniwin_Common_Functions::outside_eu_enabled();
            if ($check_outside_eu == 1) {
                woocommerce_wp_text_input(array(
                    'id'          => '_wc_shipit_uniwin_tariff_code',
                    'value'       => get_post_meta(get_the_ID(), '_wc_shipit_uniwin_tariff_code', true),
                    'label'       => __('HS tariff code', 'shipit-for-woocommerce'),
                    'placeholder' => 'Tariff code',
                    'desc_tip'    => true,
                    'description' => __('Tariff code, 6 - 10 digits', 'woocommerce'),
                    'type'        => 'text',
                ));

                woocommerce_wp_text_input(array(
                    'id'          => '_wc_shipit_uniwin_custom_description',
                    'value'       => get_post_meta(get_the_ID(), '_wc_shipit_uniwin_custom_description', true),
                    'label'       => __('Customs description', 'shipit-for-woocommerce'),
                    'placeholder' => '',
                    'desc_tip'    => true,
                    'description' => __('Description of the product for the customs.', 'woocommerce'),
                    'type'        => 'text',
                ));

                $selected_country = get_post_meta(get_the_ID(), '_wc_shipit_uniwin_custom_orgin', true);

                if (!$selected_country) {
                    $selected_country = 'FI';
                }
            ?>

                <p class="form-field _wc_shipit_uniwin_custom_orgin_field ">
                    <label for="_wc_shipit_uniwin_custom_orgin"><?php _e('Country of origin', 'shipit-for-woocommerce'); ?></label>
                    <select name="_wc_shipit_uniwin_custom_orgin" style="width: 50%;" data-placeholder="<?php esc_attr_e('Choose a country&hellip;', 'woocommerce'); ?>" class="wc-enhanced-select">
                        <?php foreach (WC()->countries->get_countries() as $country_code => $label) { ?>
                            <option value="<?php echo $country_code; ?>" <?php selected($country_code, $selected_country); ?>><?php echo $label; ?></option>
                        <?php } ?>
                    </select>
                </p> <?php
                    }
                }

                /**
                 * This code adds an action to save custom fields for a product post type. It checks if the outside EU option is enabled and if it is, it saves the values for three custom fields: tariff code, custom description, and custom origin.
                 */

                add_action('save_post_product', 'save_tariff_orgint_custom_fields', 10, 3);

                function save_tariff_orgint_custom_fields($post_id, $post, $update)
                {
                    $check_outside_eu = WC_Shipit_Uniwin_Common_Functions::outside_eu_enabled();
                    if ($check_outside_eu == 1) {
                        $attrs = array('_wc_shipit_uniwin_tariff_code', '_wc_shipit_uniwin_custom_description', '_wc_shipit_uniwin_custom_orgin');
                        foreach ($attrs as $attr) {
                            if (isset($_POST[$attr])) {
                                $value = trim($_POST[$attr]);
                                update_post_meta($post_id, $attr, $value);
                            }
                        }
                    }
                }

                /**
                 * Manual sync from order list in sync button
                 * 
                 */
                add_action('wp_ajax_uniwin_shipit_sync', 'uniwin_shipit_sync');
                function uniwin_shipit_sync()
                {
                    $order_id = $_POST['order_id'];
                    $order = wc_get_order($order_id);
                    $order_data = $order->get_data(); // The Order data

                    $shipping_methods = $order->get_shipping_methods();
                    if (empty($shipping_methods)) {
                        // The text for the note
                        $note = __("Shipit sync is failed because there is no shipping method for this order!");
                        // Add the note
                        $order->add_order_note($note);
                        echo 'failed';
                    }

                    $shipping_method = reset($shipping_methods);
                    $instance_id = FALSE;
                    if (is_a($shipping_method, 'WC_Order_Item_Shipping')) {
                        if (method_exists($shipping_method, 'get_instance_id')) {
                            $instance_id = $shipping_method->get_instance_id();
                        } else if (method_exists($shipping_method, 'get_method_id')) { // WooCommerce < 3.4
                            $shipping_rate_ids = explode(':', $shipping_method->get_method_id());
                            $shipping_rate_id = $shipping_rate_ids[0];
                            $instance_id = $shipping_rate_ids[1];
                        }
                    }

                    // No instance ID found, abort
                    if (!$instance_id) {
                        // The text for the note
                        $note = __("Shipit sync is failed because there is no shipping method instance id for this order!");
                        // Add the note
                        $order->add_order_note($note);
                        echo 'failed';
                        exit();
                    }
                    // get shipping service
                    global $wpdb;
                    $tblname = $wpdb->prefix . 'shipit_uniwin_shipping_mapping';
                    $shipping_mapped = $wpdb->get_var($wpdb->prepare("SELECT service_id FROM `" . $tblname . "` WHERE shipping_method='$instance_id'"));
                    $settings = WC_Shipit_Uniwin_Common_Functions::get_settings_info();
                    $agent_id = get_post_meta($order_id, '_shipit_uniwin_carrier_agent_id');
                    if (!empty($shipping_mapped)) {
                        //Calculate weight based on weight unit
                        $weight = WC_Shipit_Uniwin_Common_Functions::weight_calculation($order_id);
                        $_post['uniwin_shipit_country'] = $order_data['shipping']['country'];
                        $_post['uniwin_shipit_parcel'] = $settings->pk_parcel;
                        $_post['uniwin_shipit_weight'] = !empty($weight) ? $weight : 0.6;
                        $_post['uniwin_shipit_width'] = $settings->pk_width;
                        $_post['uniwin_shipit_height'] = $settings->pk_height;
                        $_post['uniwin_shipit_length'] = $settings->pk_length;
                        $_post['uniwin_shipit_service'] = $shipping_mapped;
                        $_post['uniwin_shipit_label'] = ($settings->return_label == 1) ? true : false;
                        $_post['uniwin_shipit_fragile'] = ($settings->fragile == 1) ? true : false;
                        $_post['uniwin_shipit_order_confirm'] = ($settings->order_confirm_email == 1) ? true : false;
                        $_post['uniwin_shipit_pickup'] = !empty($agent_id) ? $agent_id[0] : "";
                        $_post["oid"] = $order_id;
                        $response = WC_Shipit_Uniwin_Shipment_Functions::create_shipment($_post);
                        // Sanitize user input and update the field in the database.
                        $carrier_name = WC_Shipit_Uniwin_Shipment_Functions::all_shipit_services($shipping_mapped);
                        $data = array(
                            'package_data' => json_encode(array("country" => $order_data['shipping']['country'], "service" => $_post['uniwin_shipit_service'], "weight" => $_post['uniwin_shipit_weight'], "length" => $_post['uniwin_shipit_length'], "width" => $_post['uniwin_shipit_width'], "height" => $_post['uniwin_shipit_height'], "parcels" => $_post['uniwin_shipit_parcel'], "fragile" => isset($_post['uniwin_shipit_fragile']) ? 1 : 0, "label" => isset($_post['uniwin_shipit_label']) ? 1 : 0, "order_email" => isset($_post['uniwin_shipit_order_confirm']) ? 1 : 0)),
                            'created_at' => date('Y-m-d H:i:s'),
                            'order_id' => $order_id,
                            'shipit_response' => json_encode($response),
                            'status' => $response['status'],
                            'carrier_name' => $carrier_name
                        );
                        //Add shipment response to order meta.
                        $unique_key = 1;
                        if (!empty(get_post_meta($order_id, "wc_shipit_uniwin_shipment_response_id"))) {
                            $unique_key = get_post_meta($order_id, "wc_shipit_uniwin_shipment_response_id")[0] + 1;
                            update_post_meta($order_id, 'wc_shipit_uniwin_shipment_response_id', $unique_key);
                        } else {
                            add_post_meta($order_id, 'wc_shipit_uniwin_shipment_response_id', $unique_key, true);
                        }
                        add_post_meta($order_id, 'wc_shipit_uniwin_shipment_response_' . $unique_key, $data, true);
                        if ($response['status'] == 1) {
                            $labels = array();
                            $printing_shiping_label = WC_Shipit_Uniwin_Common_Functions::printing_shiping_label();

                            if ($printing_shiping_label == 1) {
                                // Freight docs
                                $pritinglabels = $response['freightDoc'];
                                foreach ($pritinglabels as $url) {
                                    $label = WC_Shipit_Uniwin_Common_Functions::get_printing_label_data($url);
                                    if ($label) {
                                        $labels[] = array(
                                            'type' => 'label',
                                            'contents' => $label,
                                        );
                                    }
                                }
                                // Proforma
                                if (isset($response['proforma']) && !empty($response['proforma'])) {
                                    $proforma = WC_Shipit_Uniwin_Common_Functions::get_printing_label_data($response['proforma']);
                                    if ($proforma) {
                                        $labels[] = array(
                                            'type' => 'proforma',
                                            'contents' => $proforma,
                                        );
                                    }
                                }
                            }

                            if (!empty($labels)) {
                                // Save labels to the filesystem
                                $i = 1;
                                $documents = array();
                                foreach ($labels as $label) {
                                    $document_id = sprintf("%s-%d", $response['orderId'], $i);
                                    WC_Shipit_Uniwin_Common_Functions::save_printing_label_document($label['contents'], $document_id);
                                    $documents[] = array(
                                        'id' => $document_id,
                                        'type' => $label['type'],
                                    );
                                    $i++;
                                }
                            }
                            // Print shipping label immediately after creating it
                            $redirect = false;
                            if ($printing_shiping_label == 1) {
                                if (!empty($documents) && count($documents) === 1) {
                                    $document = reset($documents);
                                    add_post_meta($order_id, 'shipit_uniwin_shipping_label', $document['id'], true);
                                    $redirect = WC_Shipit_Uniwin_Common_Functions::document_url($order_id, $document['id']);
                                } else {
                                    add_post_meta($order_id, 'shipit_uniwin_shipping_label', $documents[0]['id'], true);
                                    add_post_meta($order_id, 'shipit_uniwin_custom_docs', $documents[1]['id'], true);
                                    $redirect = WC_Shipit_Uniwin_Common_Functions::document_url($order_id, $documents[0]['id']);
                                }
                            }
                            $update_order_to_completed = WC_Shipit_Uniwin_Common_Functions::update_order_to_completed();
                            if ($update_order_to_completed == 1) {
                                $order_status = WC_Shipit_Uniwin_Common_Functions::set_order_as_completed($order_id);
                            }
                            echo 'success';
                            exit();
                        } else {
                            echo 'failed';
                            exit();
                        }
                    } else {
                        // The text for the note
                        $note = __("Shipit sync is failed because there is no mapping method!");
                        // Add the note
                        $order->add_order_note($note);
                        echo 'failed';
                        exit();
                    }
                }
            }
        }

        new WC_Shipit_Uniwin_Settings;
